module const

  implicit none
  
  real*8,parameter :: pi2 = 2.0d0*acos(-1.0d0)

end module

program langevin

implicit none

! Compile: gfortran EM.f90 -O3 -o goEM

! Name der Output-Datei
  character(len=32) :: output = 'zeitreihe.dat'

! Seed für Zufallszahlengenerator
  integer,dimension(12) :: seed
  integer,dimension(8) :: time
  
! Zeitinkrement der Simulation
  real*8,parameter :: dt = 1.0d-2

! Anzahl Zeitschritte
  integer,parameter :: nstep = 100000

! Sonstige Varaiblen

! Wurzel aus 2*Zeitinkrement
  real*8,parameter :: sqdt = sqrt(2.0d0*dt)

! Prozessvariable
  real*8 :: x

! Zufallsvariablen
  real*8,dimension(2) :: gamma

! Drift, Diffusion, Funktionen werden weiter unten definiert
  real*8 drift,diffusion

! Index
  integer i


! Zufallszahlengenerator initialisieren
  print*, 'Generate seed ...'
  call date_and_time(VALUES=time)
  
  do i=1,12
    seed(i) = time(mod(i,8)+1)
  end do

! Startwert fuer Prozess festlegen
  x = 0.0d0

  open(10,file=output,status='replace')

  do i=1,nstep

! Subroutine gauss erzeugt durch Box-Muller-Verfahren zwei Guassverteile Zufallszahlen 
    call boxmuller(gamma(1),gamma(2))
    
! Euler-Maruyama-Verfahren
    x = x + dt*drift(x) + sqdt*sqrt(diffusion(x))*gamma(1)
    
! Zeitschritt rausschreiben
    write(10,*) x
    
! Anzeigen, wie weit die Simulation ist
    if (mod(i,nstep/10).eq.0) then
      write(*,*) (i*100)/nstep, '% fertig'
    end if
    
  end do

  close(10)

end program

!==============================================================

real*8 function drift(x)

  implicit none

  real*8 x
    
  drift = -x

end function

!==============================================================

real*8 function diffusion(x)

  implicit none

  real*8 x

  diffusion = 1.0d0 

end function

!==============================================================

subroutine boxmuller(u1,u2)
  
  use const
  implicit none
  
  ! Output
  real*8 u1,u2
  
  ! Intern
  real*8 z1,z2,fac,arg
  
  call random_number(z1)
  call random_number(z2)
  
  fac = sqrt(-2.0d0*log(z1)) 
  arg = pi2*z2
  u1  = fac*cos(arg)
  u2  = fac*sin(arg)
  
end subroutine










